"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var Log_1 = require('./Log');
var Generator_1 = require('./Generator');
var t = require('./templates/DefaultNan/templates');
exports.l = new Log_1.Log(Log_1.LOGLEV.ERR);
var DefaultNanGenerator = (function (_super) {
    __extends(DefaultNanGenerator, _super);
    function DefaultNanGenerator(input, template) {
        _super.call(this, input, template);
        this.input = input;
        this.template = template;
        this.extern_c_functions = [];
        this.methods = [];
        this.inits = [];
    }
    DefaultNanGenerator.prototype.createExternDefinition = function (func) {
        var parameters = [];
        var out = "";
        func.parameters.forEach(function (param) {
            if (DefaultNanGenerator.allowedTypes.indexOf(param.type) < 0) {
                exports.l.err(param.type + " param " + param.type);
                throw Error(Generator_1.Generator.UNSUPPORTED_TYPE);
            }
            parameters.push(Generator_1.Generator.mapToCType(param.type) + " " + param.name);
        });
        if (DefaultNanGenerator.allowedTypes.indexOf(func.return) < 0) {
            exports.l.err(func.name + " returns " + func.return);
            throw Error(Generator_1.Generator.UNSUPPORTED_TYPE);
        }
        return t.externCFunc(Generator_1.Generator.mapToCType(func.return), func.name, parameters.join(", "));
    };
    /* TODO: switch to babel or typescript with proper templates otb */
    DefaultNanGenerator.prototype.createMethod = function (func) {
        var parameters = [];
        var externParams = [];
        var deallocations = [];
        var v8ReturnValue;
        var out = "";
        func.parameters.forEach(function (param, index) {
            var inType = Generator_1.Generator.mapToCType(param.type);
            switch (param.type) {
                case "c_int":
                case "c_double":
                case "c_float":
                case "bool":
                    if (param.type == "c_float") {
                        exports.l.warn(Generator_1.Generator.WARN_FLOAT);
                        inType = "double";
                    }
                    //TEMPLATE
                    parameters.push("\n  " + inType + " " + param.name + " = To<" + inType + ">(info[" + index + "]).FromJust();");
                    break;
                case "*c_char":
                    //TEMPLATE
                    parameters.push("\n  Nan::HandleScope scope;\n  String::Utf8Value cmd_" + param.name + "(info[" + index + "]);\n  string s_" + param.name + " = string(*cmd_" + param.name + ");\n  char *" + param.name + " = (char*) malloc (s_" + param.name + ".length() + 1);\n  strcpy(" + param.name + ", s_" + param.name + ".c_str());");
                    //TEMPLATE
                    deallocations.push("\n  free(" + param.name + ");");
                    break;
                default:
                    exports.l.err(param.name + " param " + param.type);
                    throw Error(Generator_1.Generator.UNSUPPORTED_TYPE);
            }
            externParams.push(param.name);
        });
        switch (func.return) {
            case "c_int":
            case "c_double":
            case "c_float":
            case "bool":
                //TEMPLATE
                v8ReturnValue = "\n  info.GetReturnValue().Set(result);";
                break;
            case "*c_char":
                //TEMPLATE
                v8ReturnValue = "\n  info.GetReturnValue().Set(Nan::New<String>(result).ToLocalChecked());\n  free(result);";
                break;
            case "void":
                break;
            default:
                exports.l.err(func.name + " returns " + func.return);
                throw Error(Generator_1.Generator.UNSUPPORTED_TYPE);
        }
        //TEMPLATE
        return ("\nNAN_METHOD(" + func.name + ") {") +
            ("" + parameters.join("")) +
            (Generator_1.Generator.mapToCType(func.return) + "\n  result = " + func.name + "(" + externParams.join(", ") + ");") +
            ("" + v8ReturnValue) +
            (deallocations.join('') + "\n}\n");
    };
    DefaultNanGenerator.prototype.createInit = function (func) {
        //TEMPLATE
        return "\n  Nan::Set(\n    target, New(\"" + func.name + "\").ToLocalChecked(),\n    Nan::GetFunction(New<FunctionTemplate>(" + func.name + ")).ToLocalChecked()\n  );";
    };
    DefaultNanGenerator.prototype.generate = function () {
        var _this = this;
        this.input.functions.forEach(function (func) {
            _this.extern_c_functions.push(_this.createExternDefinition(func));
            _this.methods.push(_this.createMethod(func));
            _this.inits.push(_this.createInit(func));
        });
        var output = this.template.replace("${extern_c_functions}", this.extern_c_functions.join(''))
            .replace("${methods}", this.methods.join(''))
            .replace("${inits}", this.inits.join(''));
        return output;
    };
    ;
    DefaultNanGenerator.allowedTypes = [
        /* some primitives from Rust's libc */
        //"c_double", //JS number
        "c_int",
        "c_float",
        "c_double",
        "bool",
        "void",
        /* pointers */
        "*c_char",
    ];
    return DefaultNanGenerator;
}(Generator_1.Generator));
exports.DefaultNanGenerator = DefaultNanGenerator;
