"use strict";
var Generator_1 = require('../../Generator');
function externCFunc(allowedTypes, func) {
    var parameters = [];
    func.parameters.forEach(function (param) {
        if (allowedTypes.indexOf(param.type) < 0) {
            Generator_1.l.err(param.type + " param " + param.type);
            throw Error(Generator_1.Generator.UNSUPPORTED_TYPE);
        }
        parameters.push(Generator_1.Generator.mapToCType(param.type) + " " + param.name);
    });
    if (allowedTypes.indexOf(func.return) < 0) {
        Generator_1.l.err(func.name + " returns " + func.return);
        throw Error(Generator_1.Generator.UNSUPPORTED_TYPE);
    }
    return "\n  extern \"C\" " + Generator_1.Generator.mapToCType(func.return) + " " + func.name + "(" + parameters.join(", ") + ");";
}
exports.externCFunc = externCFunc;
function funcParameterDefault(index, param) {
    var inType = Generator_1.Generator.mapToCType(param.type);
    if (param.type == "c_float") {
        Generator_1.l.warn(Generator_1.Generator.WARN_FLOAT);
        inType = "double";
    }
    return "\n  " + inType + " " + param.name + " = To<" + inType + ">(info[" + index + "]).FromJust();";
}
exports.funcParameterDefault = funcParameterDefault;
function funcParameterCString(index, param) {
    return "\n  Nan::HandleScope scope;\n  String::Utf8Value cmd_" + param.name + "(info[" + index + "]);\n  string s_" + param.name + " = string(*cmd_" + param.name + ");\n  char *" + param.name + " = (char*) malloc (s_" + param.name + ".length() + 1);\n  strcpy(" + param.name + ", s_" + param.name + ".c_str());";
}
exports.funcParameterCString = funcParameterCString;
function dealloc(param) {
    return "\n  free(" + param.name + ");";
}
exports.dealloc = dealloc;
function returnValueDefault() {
    return "\n  info.GetReturnValue().Set(result);";
}
exports.returnValueDefault = returnValueDefault;
function returnValueCString() {
    return "\n  info.GetReturnValue().Set(Nan::New<String>(result).ToLocalChecked());\n  free(result);";
}
exports.returnValueCString = returnValueCString;
function init(func) {
    return "\n  Nan::Set(\n    target, New(\"" + func.name + "\").ToLocalChecked(),\n    Nan::GetFunction(New<FunctionTemplate>(" + func.name + ")).ToLocalChecked()\n  );";
}
exports.init = init;
function nanMethod(func, parameters, externParams, deallocations, v8ReturnValue) {
    return ("\nNAN_METHOD(" + func.name + ") {") +
        (parameters.join("") + "\n  " + Generator_1.Generator.mapToCType(func.return) + " result = " + func.name + "(" + externParams.join(", ") + ");") +
        ("" + v8ReturnValue) +
        (deallocations.join('') + "\n}");
}
exports.nanMethod = nanMethod;
